@@ -23,6 +23,7 @@
 #include <hildon/hildon.h>
 #include <CMulticalendar.h>
 #include <Common.h>
+#include <CalendarErrors.h>
 #include "cal-home-calendar-util.h"
 
 static time_t 
@@ -184,7 +185,9 @@ cal_fill_event_list_ui(CalHomePlugin* desktop_plugin)
 			 -1);
       GtkWidget* line = gtk_hbox_new(FALSE, 0);
 
-      GtkWidget* time_label = gtk_label_new(time);
+      GtkWidget* time_label = gtk_label_new(NULL);
+
+      gtk_label_set_markup(GTK_LABEL(time_label), time);
       g_free(time);
       GtkWidget* cal_image = gtk_image_new_from_pixbuf(pixbuf);
 
@@ -195,6 +198,7 @@ cal_fill_event_list_ui(CalHomePlugin* desktop_plugin)
 				      GTK_STATE_NORMAL,
 				      "ActiveTextColor");
       GtkWidget* summary_label = gtk_label_new(NULL);
+
       gtk_label_set_markup(GTK_LABEL(summary_label), summary);
       g_free(summary);
       
@@ -262,6 +266,35 @@ _format_time(gchar* buffer, struct tm* time, gboolean has_24_time_format)
   }
 }
 
+static gboolean
+_todo_is_old(CComponent* component, struct tm* midnight)
+{
+  time_t event_time = component->getDateStart();
+  struct tm* time = localtime(&event_time);
+  return
+    (time->tm_year < midnight->tm_year) ||
+    ((time->tm_year == midnight->tm_year)  
+     && (time->tm_mon < midnight->tm_mon)) ||
+    ((time->tm_year == midnight->tm_year)  && (time->tm_mon == midnight->tm_mon) &&
+     time->tm_mday < midnight->tm_mday);
+}
+
+static void
+_format_todo_time(CComponent* component, gchar* buf, CalHomePlugin* desktop_plugin)
+{
+  time_t start_time = component->getDateStart();
+  if(cal_date_on_today(start_time, desktop_plugin))
+  {
+    desktop_plugin->today_events++;
+    sprintf(buf,"%s", dgettext("calendar", "cal_va_allday_home"));
+  }
+  else
+  {
+    struct tm* time = localtime(&start_time);
+    strftime(buf, 255,dgettext("hildon-libs","wdgt_va_date_short"), time);
+  }
+}
+
 static void
 cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent_set,
 					   GtkListStore* list_store,
@@ -281,6 +314,7 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
   time_t system_time_offset = CMulticalendar::MCInstance()->getSystemTimeShift();
 
   desktop_plugin->today_events = 0;
+
   GtkTreeIter iter;
   for(SortedCComponentSet::const_iterator event_iter = ccomponent_set.begin();
       event_iter != ccomponent_set.end() && up_to>0;
@@ -297,8 +331,18 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
       g_free(escaped_markedup);
     }
     else
-    {
-      gchar* escaped_markedup = g_markup_printf_escaped("%s", summary.c_str());
+    {      
+      struct tm today_date_today;
+      cal_home_time_today_midnight(&today_date_today);
+
+      gchar* escaped_markedup;
+      if(_todo_is_old(*event_iter, &today_date_today))
+      {
+	desktop_plugin->today_events++;
+	escaped_markedup = g_markup_printf_escaped("<span foreground=\"#ff0000\"><b>%s</b></span>", summary.c_str());
+      }
+      else
+	escaped_markedup = g_markup_printf_escaped("%s", summary.c_str());
       summary = string(escaped_markedup);
       g_free(escaped_markedup);
     }
@@ -318,9 +362,15 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
        earliest_update_end_time > event_start_time)
       earliest_update_end_time = event_start_time+60;
     char buf[255];
-
-    if(cal_date_on_today(event_start_time, desktop_plugin) || 
-       (event_start_time <= date_today_time && (event_end_time == -1 || event_end_time >= date_today_time)))
+    if((*event_iter)->getType() == E_TODO)
+    {
+      _format_todo_time(*event_iter, buf, desktop_plugin);
+      gtk_list_store_set(list_store, &iter,
+			 0, buf,
+			 -1);
+    }
+    else if(cal_date_on_today(event_start_time, desktop_plugin) || 
+	    (event_start_time <= date_today_time && (event_end_time == -1 || event_end_time >= date_today_time)))
     {
       ++desktop_plugin->today_events;
 
@@ -342,7 +392,11 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
     else
     {
       if(desktop_plugin->show_week_day_name)
-	strftime(buf, 255,dgettext("hildon-libs","wdgt_va_date_day_name_short"), time);
+      {
+	gchar* tt = g_strdup_printf("<tt>%s</tt>", dgettext("hildon-libs","wdgt_va_date_day_name_short"));
+	strftime(buf, 255, tt, time);
+	g_free(tt);
+      }
       else
 	strftime(buf, 255,dgettext("hildon-libs","wdgt_va_date_short"), time);
       gtk_list_store_set(list_store, &iter,
@@ -359,8 +413,6 @@ cal_condense_ccomponents_set_to_list_store(const SortedCComponentSet& ccomponent
     g_object_unref(pixbuf);
     --up_to;
   }
-  struct tm now;
-  cal_home_time_today_now(&now);
   
   if(desktop_plugin->time_out_handler)
   {
@@ -382,6 +434,63 @@ cal_put_age_on_summary(const int& born,
   return new_summary;
 }
 
+static void
+cal_read_todos_for_calendar(CCalendar* calendar,
+			    time_t start_time,
+			    time_t end_time,
+			    SortedCComponentSet& all_components,
+			    CalHomePlugin* desktop_plugin)
+{
+  int error = 0;
+  int offset = 0;
+  int size = 0;
+  int limit = 200;
+  struct tm today_midnight; 
+  cal_home_time_today_midnight(&today_midnight);
+
+  do
+  {
+    vector<CComponent*> calendars_components = calendar->getComponents(E_TODO,
+								       start_time,
+								       end_time,
+								       limit,
+								       offset,
+								       error);
+    size = calendars_components.size();
+    offset+=200;
+    limit+=200;
+    time_t last_start_date = start_time;
+    for(int k=0;k<calendars_components.size();++k)
+    {
+      CComponent* component = calendars_components[k];
+      gchar buf[256];
+      time_t event_time = component->getDateStart();
+      struct tm* time = localtime(&event_time);
+      if(_todo_is_old(component, &today_midnight))
+      {
+	if((component->getStatus() == 0) &&
+	   (desktop_plugin->show_old_undone_tasks))
+	{
+	  all_components.insert(component);
+	}
+	else
+	{
+	  delete component;
+	}
+      }
+      else if((component->getStatus() == 0) ||
+	      desktop_plugin->show_done_tasks)
+      {
+	all_components.insert(component);
+      }
+      else
+      {
+	delete component;
+      }
+    }
+  }while(size != 0);
+}
+
 static void
 cal_read_ccompnents_for_calendar(CCalendar* calendar,
 				 time_t start_time,
@@ -393,16 +502,26 @@ cal_read_ccompnents_for_calendar(CCalendar* calendar,
   int error = 0;
   int offset = 0;
   int size = 0;
-  int limit = 100;
+  int limit = 500;
   do
   {
-    vector<CComponent*> calendars_components = calendar->getAllEventsTodos(start_time,
-									   end_time,
-									   limit,
-									   offset,
-									   error);
+    int event_type = E_EVENT;
+    if(birthday_only)
+      event_type = E_BDAY;
+    vector<CComponent*> calendars_components = calendar->getComponents(event_type,
+								       start_time,
+								       end_time,
+								       limit,
+								       offset,
+								       error);
+    /* vector<CComponent*> calendars_components = calendar->getAllEventsTodos(start_time, */
+    /* 									   end_time, */
+    /* 									   limit, */
+    /* 									   offset, */
+    /* 									   error); */
     size = calendars_components.size();
-    offset+=limit;
+    offset=limit+1;
+    limit+=500;
     time_t last_start_date = start_time;
     time_t system_time_offset = CMulticalendar::MCInstance()->getSystemTimeShift();
     for(int k = 0;k<calendars_components.size();++k)
@@ -411,9 +530,12 @@ cal_read_ccompnents_for_calendar(CCalendar* calendar,
 
       time_t event_time_offset = component->getTzOffset();
 
+      //      vector<time_t> instance_times;
+
       vector<time_t> instance_times =
 	component->getInstanceTimes(last_start_date, 
 				    end_time);
+
       if((desktop_plugin->visible_entry_mode == VISIBLE_ENTRY_EVENT && calendars_components[k]->getType() == E_TODO) ||
 	 (FALSE==birthday_only && 
 	  ((calendars_components[k]->getType() == E_BDAY) ||
@@ -457,7 +579,7 @@ cal_read_ccompnents_for_calendar(CCalendar* calendar,
 	}
       }
     }
-  }while(size !=0 );
+  }while(error == CALENDAR_OPERATION_SUCCESSFUL);
 }
 
 static void
@@ -479,17 +601,40 @@ cal_read_ccomponents_all_visible(SortedCComponentSet& all_components,
     if(error == 500)
     {
       cal_read_ccompnents_for_calendar(cal,
-				       start_time+1,
+				       start_time,
 				       end_time,
 				       all_components, FALSE,
 				       desktop_plugin);
       // gnark, birthday events start and end
       // at 00:00:00
+
       cal_read_ccompnents_for_calendar(cal,
 				       today_midnight_time,
 				       end_time,
 				       all_components, TRUE,
 				       desktop_plugin);
+
+
+      if(desktop_plugin->visible_entry_mode == VISIBLE_ENTRY_TASK ||
+	 desktop_plugin->visible_entry_mode == VISIBLE_ENTRY_EVENT_AND_TASK)
+      {
+	if(desktop_plugin->show_old_undone_tasks)
+	{
+	  cal_read_todos_for_calendar(cal,
+				      -1,
+				      -1,
+				      all_components,
+				      desktop_plugin);
+	}
+	else
+	{
+	  cal_read_todos_for_calendar(cal, 
+				      today_midnight_time, 
+				      end_time, 
+				      all_components, 
+				      desktop_plugin); 
+	}
+      }
     }
     delete cal;
   }
